@@ -1,15 +1,15 @@
 use std::borrow::Cow;
+use std::collections::BTreeSet;
 
-use deser::ser::for_each_event;
+use deser::ser::SerializeDriver;
 use deser::{Atom, Event, Serialize};
 
 fn capture_events(s: &dyn Serialize) -> Vec<Event<'static>> {
     let mut events = Vec::new();
-    for_each_event(s, |event, _, _| {
+    let mut driver = SerializeDriver::new(s);
+    while let Some((event, _, _)) = driver.next().unwrap() {
         events.push(event.to_static());
-        Ok(())
-    })
-    .unwrap();
+    }
     events
 }
 
@@ -82,3 +82,15 @@ fn test_box() {
     let events = capture_events(&Box::new(true));
     assert_eq!(events, vec![true.into()]);
 }
+
+#[test]
+fn test_set() {
+    let mut set = BTreeSet::new();
+    set.insert("foo");
+    set.insert("bar");
+    let events = capture_events(&set);
+    assert_eq!(
+        events,
+        vec![Event::SeqStart, "bar".into(), "foo".into(), Event::SeqEnd]
+    );
+}
